  
# Credit goes to Daniel J. Berger, via PTools.
# The nl_convert() method was adopted (somewhat) from the nlcvt program
# found at http://www.perl.com/language/ppt/src/nlcvt/nlcvt, written by
# Tom Christiansen.

class FileUitls
  
  # Converts a text file from one OS platform format to another, ala
  # ((*dos2unix*)).  Valid values for ((*format*)), which are case insensitve,
  # includes:
  #
  # * MS Windows -> dos, windows, win32
  # * Unix/BSD   -> unix, linux, bsd
  # * Mac        -> mac, macintosh, apple, osx
  #
  # Note that this method is only valid for an ftype of "file".  Otherwise a
  # TypeError will be raised.  If an invalid format value is received,
  # StandardError is raised.
  #
  #   # Creates a copy of 'myfile' called 'newfile', in DOS format
  #   File.nl_convert("myfile","newfile","dos")
  #
  # (Note: Looking to improve, especially getting rid of using ftools.)
  
  def self.nl_convert(old_file,new_file=old_file,platform="dos")

    unless File.ftype(old_file) == "file"
        raise TypeError, "Only valid for plain text files"
    end

    if platform =~ /dos|windows|win32/i
        format = "\cM\cJ"
    elsif platform =~ /unix|linux|bsd/i
        format = "\cJ"
    elsif platform =~ /mac|apple|macintosh|osx/i
        format = "\cM"
    else
        raise StandardError, "Invalid platform string"
    end

    orig = $\
    $\ = format

    if old_file == new_file
        require "ftools"
        require "tempfile"
        tf = Tempfile.new("temp")
        tf.open
        IO.foreach(old_file){ |line|
          line.chomp!
          tf.print line
        }
        tf.close
        File.delete(old_file)
        File.copy(tf.path,old_file)
    else
        nf = File.new(new_file,"w+")  
        IO.foreach(old_file){ |line|
          line.chomp!
          nf.print line
        }
        nf.close
    end

    $\ = orig
  end

end
