
class String

  # --- str.prev      => string
  # Minimal difference previous.
  # #prev is the essentially the opposite of #next. 
  # #next and #prev are not exact opposites however,
  # as some cases of #prev are indeterminate. In these
  # cases #prev returns the minimal possible solution.
  #
  #   "123ABC".prev  #=> "123ABB"
  #
  # Credit goes to Pit Capitian.
  # (The code could use some improving.)
  def prev
    case self
    when ''
      ''
    when /[1-9B-Zb-z]/
      alnum_pred
    when /[0Aa].*[0Aa]/
      alnum_pred
    when /[0Aa]\000*$/
      binary_pred
    when /[0Aa]/
      new_pred
    else
      binary_pred
    end
  end
  
  # ---- str.pred      => string
  # Minimal difference predecessor. It is the essentially the opposite
  # of #succ, althouth it is not the exact opposite. This version adds
  # an optional paramter to specify the _n_th predecessor be returned
  def pred(n=1)
    s = self
    n.times { s = s.prev }
    s
  end
  
  alias_method( :pred_once, :prev )
  
  private
  
    def alnum_pred
      result = dup
      zero_idx = last_idx = borrowed = borrowed_chars = nil
      idx = size
      while idx > 0
        idx -= 1
        last_idx = idx unless last_idx
        case result[ idx ]
        when ?1 .. ?9, ?B .. ?Z, ?b .. ?z
          result[ idx ] -= 1
          zero_idx = idx if result[ idx ] == ?0
          borrowed = nil
          break
        when ?0
          borrowed = result[ idx ] = ?9
          borrowed_chars = 1
        when ?A, ?a
          result[ idx ] += ( ?Z - ?A )
          zero_idx = idx
          if borrowed == result[ idx ]
            borrowed_chars -= 1
          else
            borrowed = result[ idx ]
            borrowed_chars = 1
          end
        else
          last_idx = nil if last_idx == idx
        end
      end
      if borrowed && borrowed_chars > 0
        result = new_pred
      elsif zero_idx && zero_idx < last_idx
        idx = 0
        loop do
          case result[ idx ]
          when ?1 .. ?9, ?A .. ?Y, ?a .. ?y
            break
          when ?0, ?Z, ?z
            result[ idx, 1 ] = '' if idx == zero_idx
            break
          else
            idx += 1
          end
        end
      end
      result
    end
    
    def binary_pred
      result = dup
      idx = size
      while idx > 0
        idx -= 1
        if result[ idx ] > 0
          result[ idx ] -= 1
          break
        else
          result[ idx ] = 255
        end
      end
      if idx == 0
        case result[ idx ]
        when 0
          result[ 0, 1 ] = '' if size > 1
        when 255
          result = new_pred
        end
      end
      result
    end
    
    def new_pred
      "new_pred " + self
    end
  
end


