
require 'test/unit'

require 'facet/string/pred'


class Pred_TC < Test::Unit::TestCase

  def assert_pred( pred, str, msg = str )
    assert_equal( pred, str.pred, "pred #{msg}" )
  end

  def assert_inverse( str, msg = str )
    assert_equal( str, str.pred.succ, "inverse #{msg}" )
  end

  def assert_pred_with_inverse( pred, str, msg = str )
    assert_pred( pred, str, msg )
    assert_inverse( str, msg )
  end

  def test_01_empty
    assert_pred_with_inverse( '', '', 'empty' )
  end

  def test_02_simple_alnum
    assert_pred_with_inverse( '1', '2' )
    assert_pred_with_inverse( '42', '43' )
    assert_pred_with_inverse( '3999', '4000' )
    assert_pred_with_inverse( '00399', '00400' )
    assert_pred_with_inverse( 'A', 'B' )
    assert_pred_with_inverse( 'HH', 'HI' )
    assert_pred_with_inverse( 'AZZ', 'BAA' )
    assert_pred_with_inverse( 'AAAZZ', 'AABAA' )
    assert_pred_with_inverse( 'd', 'e' )
    assert_pred_with_inverse( 'xx', 'xy' )
    assert_pred_with_inverse( 'yzz', 'zaa' )
    assert_pred_with_inverse( 'zzyzz', 'zzzaa' )
    assert_pred_with_inverse( '8Z9z', '9A0a' )
    assert_pred_with_inverse( ' /4-Z++9 $z #9#', ' /5-A++0 $a #0#' )
  end

  def test_03_leading_1Aa
    assert_pred_with_inverse( '0', '1' )
    assert_pred_with_inverse( '99', '100' )
    assert_pred_with_inverse( '00099', '00100' )
    assert_pred_with_inverse( 'Z', 'AA' )
    assert_pred_with_inverse( 'zZz', 'aaAa' )
    assert_pred_with_inverse( '9zZ', '10aA' )
    assert_pred_with_inverse( 'Z9z9', 'AA0a0' )
    assert_pred_with_inverse( ' /9-Z++9 $z #9#', ' /10-A++0 $a #0#' )
  end

  def test_04_illegal_alnum
    assert_pred( 'new_pred 00', '00' )
    assert_pred( 'new_pred 0000', '0000' )
    assert_pred( 'new_pred A0', 'A0' )
    assert_pred( 'new_pred 0a0', '0a0' )
    assert_pred( 'new_pred 00Aa', '00Aa' )
    assert_pred( 'new_pred aA', 'aA' )
    assert_pred( 'new_pred Aa00', 'Aa00' )
  end

  def test_05_simple_binary
    assert_pred_with_inverse( ' ', '!' )
    assert_pred_with_inverse( '!$', '!%' )
    assert_pred_with_inverse( "$\377\377\377", "%\000\000\000" )
    assert_pred_with_inverse( "<==\377", "<=>\000" )
    assert_pred_with_inverse( "\377 ", "\377!" )
  end

  def test_06_leading_001
    assert_pred_with_inverse( "\000", "\001" )
    assert_pred_with_inverse( "\377", "\001\000" )
    assert_pred_with_inverse( "\377\377\377", "\001\000\000\000" )
    assert_pred_with_inverse( "\000\000\377", "\000\001\000" )
  end

  def test_07_binary_with_0Aa
    assert_pred_with_inverse( '/', '0' )
    assert_pred_with_inverse( '@', 'A' )
    assert_pred_with_inverse( '`', 'a' )
    assert_pred_with_inverse( "$ @\377", "$ A\000" )
    assert_pred_with_inverse( "+/\377\377", "+0\000\000" )
  end

  def test_08_illegal_binary
    assert_pred( "new_pred \000", "\000" )
    assert_pred( "new_pred \000\000", "\000\000" )
    assert_pred( "new_pred 0\001", "0\001" )
    assert_pred( "new_pred A+", "A+" )
    assert_pred( "new_pred <a>", "<a>" )
  end

end
 
