class Array

  # Return random _number_ of value(s) from an Array. If the
  # _number_ of elements is not specified (or is <tt>nil</tt>)
  # then returns a single element, otherwise returns a random
  # array of elements. By defualt the values are exclusive of
  # each other, but if _exclusive_ is set to <tt>false</tt>, the
  # same values can be choosen more than once. 
  #
  # When _exclusive_ is <tt>true</tt> (the default) and the 
  # _number_ given is greater than the size of the array, 
  # then all values are returned.
  #
  #   require 'facet/array/at_rand'
  #
  #   [1, 2, 3, 4].at_rand           #=> 2
  #   [1, 2, 3, 4].at_rand(1)        #=> [2]
  #   [1, 2, 3, 4].at_rand(2)        #=> [1,4]
  #   [1, 2, 3, 4].at_rand(3)        #=> [4,3,2]
  #   [1, 2, 3, 4].at_rand(3,false)  #=> [4,4,2]
  #
  def at_rand( number=nil, exclusive=true )
    unless number
      return at( rand_index )
    else
      number = number.to_int
      values_at(*rand_index(number,exclusive))
    end
  end

  # Return random index(es) from an Array. If the _number_
  # of elements is not specified (or is <tt>nil</tt>) then
  # returns just a single index.
  #
  # When _exclusive_ is <tt>true</tt> (the default) and the 
  # _number_ given is greater than the size of the array, 
  # then all indexes are returned.
  #
  #   require 'facet/array/at_rand'
  #
  #   ["a", "b", "c", "d"].rand_index           #=> 1
  #   ["a", "b", "c", "d"].rand_index           #=> 2
  #   ["a", "b", "c", "d"].rand_index(2)        #=> [2,1]
  #   ["a", "b", "c", "d"].rand_index(3)        #=> [1,3,2]
  #   ["a", "b", "c", "d"].rand_index(3,false)  #=> [4,3,4]
  #
  def rand_index( number=nil, exclusive=true )
    unless number
      rand(size)
    else
      number = number.to_int
      if exclusive  
        idx = (0...size).to_a
        return idx if number >= size
        return idx.sort_by{rand}.slice(0,number)
      else
        a = []
        n.times{ a << rand(size) }
        a
      end
    end
  end

  # Same as #at_rand, but acts in place removing the
  # element from the array. This is necessarily exclusive.
  #
  #   require 'facet/array/at_rand'
  #
  #   a = [1,2,3,4]
  #   a.at_rand!       #=> 2
  #   a                #=> [1,3,4]
  #
  #   a = [1,2,3,4]
  #   a.at_rand!(2)    #=> [2,1]
  #   a                #=> [3,4]
  #
  def at_rand!(number=nil)
    unless number
      return delete_at( rand_index )
    else
      number = number.to_int
      all = (0...size).to_a
      idx = rand_index(number)
      vals = values_at(*idx)
      replace( values_at( *(all - idx) ) )
      return vals    
    end
  end
  alias_method( :pick, :at_rand! )
 
end


#--
# SEEMS RATHER USELESS
#
#   # Destructive rand_index.  Delete a random element of 
#   # an array and return its index.
#   #
#   #   require 'facet/array/at_rand'
#   #
#   #   a = [11, 22, 33, 44]
#   #   a.rand_index!  #=> 2
#   #   a              #=> [11, 22, 44]
#   #
#   def rand_index!(n=nil)
#     unless n
#       i = rand_index
#       delete_at(i)
#       i
#     else
#       idx = rand_index(n)
#       idx.uniq.each{ |i| self.delete_at(i) }
#       return idx
#     end
#   end
#
#++
