class Array 
  # Rotates an array's elements from back to front n times.
  #
  #   require 'facet/array/rotate'
  #
  #   [1,2,3].rotate      #=> [3,1,2]
  #   [3,1,2].rotate      #=> [2,3,1]
  #   [3,1,2].rotate      #=> [1,2,3]
  #   [1,2,3].rotate(3)   #=> [1,2,3]
  #
  # A negative parameter reverses the order from front to back.
  #
  #   [1,2,3].rotate(-1)  #=> [2,3,1]
  #
  def rotate(n=1)
    self.dup.rotate!(n)
  end
  
  # Same as #rotate, but acts in place.
  #
  #   require 'facet/array/rotate'
  #
  #   a = [1,2,3]
  #   a.rotate
  #   a  #=> [3,1,2]
  #
  def rotate!(n=1)
    n = n.to_i
    return self if n == 0
    if n > 1
      n.times{ unshift(pop) }
    else
      n.times{ push(shift) }
    end
    self
  end
  
end
