module Enumerable
  # Same as #collect but with an iteration counter.
  #
  #   require 'facet/enumerable/collect_with_counter'
  #
  #   a = [1,2,3].collect_with_counter{ |e,i| e*i }
  #   a  #=> [0,2,6]
  #
  #--
  # Why the term counter? There may be a change in Ruby 2.0
  # to use this word instead of index --while index will
  # still be used for Array, since theat is the proper meaning
  # in that context. In the mean time, aliases are provided.
  #++
  def collect_with_counter
    r = []
    each_index do |i|
      r << yield(self[i], i)
    end
    r
  end
  alias_method( :map_with_counter, :collect_with_counter )
  alias_method( :collect_with_index, :collect_with_counter )
  alias_method( :map_with_index, :collect_with_counter )
end
