#--
# Cross product adapted from code by Michael Neuman.
#++
require 'generator'

module Enumerable

  #  Provides the cross-product of two or more Enumerables.
  #  This is the class-level method. The instance method
  #  calls on this.
  #
  #    require 'facet/enumerable/cross'
  #
  #    Enumerable.cross([1,2], [4], ["apple", "banana"])
  #    #=> [[1, 4, "apple"], [1, 4, "banana"], [2, 4, "apple"], [2, 4, "banana"]]
  #
  #    Enumerable.cross([1,2], [3,4])
  #    #=> [[1, 3], [1, 4], [2, 3], [2, 4]]
  #
  def self.cross(*enums, &block)
    raise if enums.empty?
    gens = enums.map{|e| Generator.new(e)}
    return [] if gens.any? {|g| g.end?}
    sz = gens.size
    res = []
    tuple = Array.new(sz)

    loop do
      # fill tuple
      (0 ... sz).each { |i| 
        tuple[i] = gens[i].current 
      }
      if block.nil?
        res << tuple.dup
      else
        block.call(tuple.dup)
      end

      # step forward
      gens[-1].next
      (sz-1).downto(0) { |i|
        if gens[i].end?
          if i > 0
            gens[i].rewind
            gens[i-1].next
          else
            return res
          end
        end
      }
    end
  end
  
  # The instance level cross-product method.
  #
  #   require 'facet/enumerable/cross'
  #
  #   a = []
  #   [1,2].cross([4,5]){|elem| a << elem }
  #   a  #=> [[1, 4],[1, 5],[2, 4],[2, 5]]
  #  
  def cross(*enums, &block)
    Enumerable.cross(self, *enums, &block)
  end
  
end


=begin
# --- dev testing ---

if __FILE__ == $0
  p Enumerable.cross([1,2], [4], ["apple", "banana"])

  [1,2,3].cross([4,5,6]) {|elem| p elem }

  p Enumerable.cross([1,2], [3,4])
end
=end
