module Enumerable
  # Iterate through slices. If slicing _step_ is not
  # given, the the arity if the block is used.
  #
  #   require 'facet/enumerable/each_slice'
  #
  #   x = []
  #   [1,2,3,4].each_slice(2){ |a,b| x << [a,b] }
  #   x  #=> [ [1,2], [3,4] ]
  #
  #   x = []
  #   [1,2,3,4,5,6].each_slice(3){ |a| x << a }
  #   x  #=> [ [1,2,3], [4,5,6] ]
  #
  def each_slice(step=nil, &yld)
    if step
      cnt = step
      n = length / cnt
      n += 1 if length % cnt > 0
      n.times { |i|
        yld.call(*self.slice(i*cnt,cnt))
      }
      return cnt
    else
      cnt = yld.arity.abs
      n = length / cnt
      n += 1 if length % cnt > 0
      n.times { |i|
        yld.call(*self.slice(i*cnt,cnt))
      }
      return cnt
    end  
  end
  
  # Alias for #each_slice.
  alias_method( :each_by, :each_slice )
end


class Array
  # Iterates over +n+ elements at a time. If +n+ is
  # not given then the arity of the block determines the 
  # slicing quantity.
  #
  #   require 'facet/array/each_slice'
  #
  #   [1, 2, 3, 4].each_slice(2){ |a,b| ... }
  #
  # This is a specialized version Enumerable#each_slice
  # but optimized specifically for Array.
  def each_slice(n=nil, &yld)
    n = yld.arity.abs unless n
    i=0
    while i < self.length
      yld.call(*self.slice(i,n))
      i+=n
    end
  end
  alias_method( :each_by, :each_slice )
end
