#--
# Credit goes to Gavin Sinclair.
#++
module Enumerable
  # See Enumerable#partition for the background.  #partition_by is best
  # explained by example.
  #
  #   require 'facet/enumerable/partition_by'
  #
  #   (1..5).partition_by { |n| n % 3 }
  #        #=> { 0 => [3], 1 => [1, 4], 2 => [2,5] } 
  #
  #   ["I had", 1, "dollar and", 50, "cents"].partition_by { |e| e.class }
  #        #=> { String => ["I had","dollar and","cents"], Fixnum => [1,50] } 
  #
  # #partition_by is used to group items in a collection by something they
  # have in common.  The common factor is the key in the resulting hash, the
  # array of like elements is the value.
  def partition_by
    result = {}
    self.each do |e|
      value = yield e
      (result[value] ||= []) << e
    end
    result
  end
end 
