module Enumerable
  
  # Like #uniq, but determines uniqueness based
  # on a given block.
  #
  #   require 'facet/enumerable/uniq_by'
  #
  #   (-5..5).to_a.uniq_by {|i| i*i }
  #
  # produces
  #
  #   [-5, -4, -3, -2, -1, 0]
  #
  def uniq_by
    h = {}; inject([]) {|a,x| h[yield(x)] ||= a << x} 
  end

  # # alternate definition
  # #
  # #  [[1, 2], [3, 4], [5, 6], [7, 2]].uniq_by(:at, 1)
  # #
  # def uniq_by(*args, &blk)
  #   blk = lambda {|i| i.send(*args)} unless args.empty?
  #   raise "needs args or block" unless blk
  #   h = {}
  #   a = []
  #   each {|i|
  #     k = blk.call(i)
  #     a << i unless h[k]
  #     h[k] = true
  #   }
  #   a
  # end

end
