#--
# The body() method was provided by Shashank Date, via Daniel Berger.
#++
require 'fileutils'

module FileUtils

  module_function

  # In block form, yields the first number of ((*lines*)) of file ((*filename*)).
  # In non-block form, it returns an array of the first number of ((*lines*)).
  #
  #   require 'facet/fileutils/head&tail'
  #
  #   # Returns first 10 lines of 'myfile'
  #   FileUtils.head("myfile")
  #  
  def head(filename,lines=10) #:yield:
    a = []
    IO.foreach(filename){|line|
        break if lines <= 0
        lines -= 1
        if block_given?
          yield line
        else
          a << line
        end
    }
    return a.empty? ? nil : a
  end
  #alias first head

  # In block form, yields lines ((*from*))-((*to*)).  In non-block form,
  # returns an array of lines ((*from*))-((*to*)).
  #
  #   require 'facet/fileutils/head&tail'
  #
  #   # Returns lines 8-12 of 'myfile'
  #   FileUtils.body("myfile",8,12)
  #
  def body(filename,from=10,to=20) #:yield:
    IO.readlines(filename)[from-1..to-1]
  end
  #alias middle body
  #alias mid body
    
  # In block form, yields the last number of ((*lines*)) of file ((*filename*)).
  # In non-block form, it returns the lines as an array.
  #
  # Note that this method slurps the entire file, so I don't recommend it
  # for very large files.  If you want an advanced form of ((*tail*)), I
  # suggest using file-tail, by Florian Frank (available on the RAA).
  # And no tail -f.
  #
  #   require 'facet/fileutils/head&tail'
  #
  #   # Returns last 3 lines of 'myfile'
  #   FileUtils.tail("myfile",3)
  #
  def tail(filename,lines=10) #:yield
    IO.readlines(filename).reverse[0..lines-1].reverse
  end 
  #alias last tail
  
end
