class Float
  # Rounds off at the given decimal position.
  def round_at( d=0.0 )
    (self * (10.0 ** d)).round.to_f / (10.0 ** d)
  end
  # Rounds to the nearest _n_th. If no _n_th is
  # given, then defaults to nearest 100th.
  def round_to( n=0.01 )
    raise ArgumentError, "zero equates to infinte degree" if n == 0
    (self * (1.0 / n)).round.to_f / (1.0 / n)
  end
  # Determines if another number is approximately equal
  # within a given _n_th degree. Defaults to 100ths
  # if degree is not specified.
  def approx?( x, n=0.01 )
    raise ArgumentError, "zero equates to infinte degree" if n == 0
    self.round_to(n) == x.to_f.round_to(n)
  end
end

# To properly support Float rounding methods,
# Numeric must also be augmented.
class Numeric
  def round_at(*args)
    self.to_f.round_at(*args)
  end
  def round_to(*args)
    self.to_f.round_to(*args)
  end
  def approx?(*args)
    self.to_f.approx?(*args)
  end
end
