#--
# Adapted from Gavin Sinclair's #convert_keys.
#++
class Hash
  
  # Converts all keys in the Hash to be String values, returning a new Hash.
  # With a from_class parameter, limits conversion to only a certain class of keys.
  # It defaults to nil which convert any key class.
  #
  #   require 'facet/hash/keys_to_string'
  #
  #   foo = { :name=>'Gavin', :wife=>:Lisa }
  #   foo.to_s_keys    #=>  { "name"=>"Gavin", "wife"=>:Lisa }
  #   foo.inspect      #=>  { :name =>"Gavin", :wife=>:Lisa }
  #
  def keys_to_s( from_class=nil )
    raise ArgumentError, "Parameter must be a class" unless from_class.kind_of?(Class) if from_class
    self.dup.keys_to_s!( from_class )
  end
  
  
  # Synonym for Hash#keys_to_string, but modifies the receiver in place (and returns it).
  # With a from_class parameter, limits conversion to only a certain class of keys.
  # It defaults to nil which convert any key class.
  #
  #   require 'facet/hash/keys_to_string'
  #
  #   foo = { :name=>'Gavin', :wife=>:Lisa }
  #   foo.to_s_keys!    #=>  { "name"=>"Gavin", "wife"=>:Lisa }
  #   foo.inspect       #=>  { "name"=>"Gavin", "wife"=>:Lisa }
  #
  def keys_to_s!( from_class=nil )
    raise ArgumentError, "Parameter must be a class" unless from_class.kind_of?(Class) if from_class
    if from_class
      self.each_key{ |k| self[k.to_s]=self.delete(k) if k.respond_to?(:to_s) and k.class == from_class }
    else
      self.each_key{ |k| self[k.to_s]=self.delete(k) if k.respond_to?(:to_s) }
    end
    self
  end

  # For Ruby/Extensions compatibility.
  alias_method( :convert_keys, :keys_to_s )
  alias_method( :convert_keys!, :keys_to_s! )

  # These are deprecated.
  #alias_method( :keys_to_string, :keys_to_s )
  #alias_method( :keys_to_string!, :keys_to_s! )
  
end
