#--
# Adapted from rand.rb by:
# * Ilmari Heikkinen <mailto:kig@misfiring.net>
# * Christian Neukirchen <mailto:chneukirchen@gmail.com>
#++
class Hash

  # Return the key-value pairs with _keys_ and _values_
  # shuffled independedly.
  #
  #   require 'facet/hash/shuffle'
  #
  #   {:a=>1, :b=>2, :c=>3}.shuffle_hash_pairs  
  #
  # produces
  #
  #   [[:a, 3], [:b, 1], [:c, 2]]
  #
  def shuffle_hash_pairs
    keys.shuffle.zip(values.sort_by{Kernel.rand})
  end

  # Returns a copy of the hash with _values_ arranged
  # in new random order.
  #
  #   require 'facet/hash/shuffle'
  #
  #   h = {:a=>1, :b=>2, :c=>3}
  #   h.shuffle_hash  #=> {:b=>2, :c=>1, :a>3}
  #
  def shuffle_hash
    shuffled = {}
    shuffle_hash_pairs.each{|k, v| shuffled[k] = v }
    shuffled
  end

  # Destructive shuffle_hash.  Arrange the values in 
  # a new random order.
  #
  #   require 'facet/hash/shuffle'
  #
  #   h = {:a => 1, :b => 2, :c => 3}
  #   h.shuffle_hash!
  #   h  #=> {:b=>2, :c=>1, :a=>3}
  #
  def shuffle_hash!
    shuffle_hash_pairs.each{ |k, v| self[k] = v }
    self
  end

end
