class Hash
  # Iterates through each pair and updates a the hash 
  # in place. This is formally equivalent to #collate!
  # But does not use #collate to accomplish the task.
  # Hence #update_each is probably a bit faster.
  #
  #   require 'facet/hash/update_each'
  #
  #   # example to do
  #
  #--
  # Note that this may get some fine tuning as currently
  # it expects the block to return a "mini-hash" pair.
  #++
  def update_each  # :yield:
    dup.each_pair{ |k,v| update( yield(k,v) ); }
    self
  end
  # Same as #update_each, but deletes the key element first.
  def replace_each  # :yield:
    dup.each_pair{ |k,v| delete( k ); update( yield(k,v) ); }
    self
  end
end
