#--
# Credit goes to Richard Kilmer.
# And thanks to Ruby On Rails for making these readily available.
# And thanks to Alexander Kellett for suggesting that Facet's offer them.
# Also thanks to Francis Hwang for suggesting improvements.
#++

class Numeric
  
  # Converts microseconds into a fraction of seconds.
  def microseconds
    self.to_f / 1000000.0
  end
  alias :microsecond :microseconds
  
  # Converts milliseconds into a fraction of seconds.
  def milliseconds
    self.to_f / 1000.0
  end
  alias :millisecond :milliseconds

  # Enables the use of time calculations and declarations, 
  # like 45.minutes + 2.hours + 4.years. The base unit for
  # all of these Numeric time methods is seconds.
  def seconds
    self
  end
  alias :second :seconds
  
  # Converts minutes into seconds.
  def minutes
    self * 60
  end
  alias :minute :minutes
  
  # Converts hours into seconds.
  def hours
    self * 60.minutes
  end
  alias :hour :hours
  
  # Converts days into seconds.
  def days
    self * 24.hours
  end
  alias :day :days

  # Converts weeks into seconds.
  def weeks
    self * 7.days
  end
  alias :week :weeks
  
  # Converts fortnights into seconds.
  # (A fortnight is 2 weeks)
  def fortnights
    self * 2.weeks
  end
  alias :fortnight :fortnights
  
  # Converts months into seconds.
  # WARNING: This is not exact as it assumes 30 days to a month.
  def months
    self * 30.days
  end
  alias :month :months
  
  # Converts years into seconds.
  def years
    self * 365.days
  end
  alias :year :years

  # Calculates time _before_ a given time. Default time is now.
  # Reads best with arguments: 10.days.before( Time.now - 1.day )
  def before(time = ::Time.now)
    time - self
  end

  # Alias for #before.
  # Reads best with argument: 10.minutes.until(time)
  alias_method :until, :before

  # Alias for #before.
  # Reads best without arguments: 10.minutes.ago
  alias_method :ago, :before

  # Calculates time _after_ a given time. Default time is now.
  # Reads best with argument: 10.minutes.after(time)
  def after(time = ::Time.now)
    time + self
  end

  # Alias for #after.
  # Reads best with argument: 10.minutes.since(time)
  alias_method :since, :after

  # Alias for #after.
  # Reads best without arguments: 10.minutes.from_now
  alias_method :from_now, :after

  # Alias for #after.
  # Reads best without arguments: 10.minutes.later
  alias_method :later, :after

end
