class Object
  
  # Set setter methods and/or vars using a hash (or assoc array).
  # #assign_with is a meta-programming method, which allows you to 
  # use a hash to do any kind of variable assignment and/or setting:
  #
  # For example, assuming that there is an accessor defined as <tt>d</tt>:
  #
  #   require 'facet/object/assign_with'
  #
  #   assign_with { '@a'=>1, '@@b'=>2, '$c'=>3, 'd'=>4 }
  #   @a   #=> 1
  #   @@b  #=> 2
  #   $c   #=> 3
  #   d    #=> 4
  #
  # Note that while the global variable is strictly unnecessary,
  # it works for completeness sake.
  def assign_with(*args)
    if args[0].is_a?(Hash) #or args[0].is_a?(Array)
      iv_val_pairs = args[0]
    else
      raise ArgumentError, "Hash required for assigning multiple variables" if args.length > 2
      iv_val_pairs = { args[0] => args[1] }
    end
    iv_val_pairs.each_pair do |k,v|
      if k.to_s.slice(0,2) == '@@'
        self.instance_eval %Q{ #{k} = v }
      else
        case k.to_s.slice(0,1)
        when '$'
          self.instance_eval %Q{ #{k} = v }
        when '@'
          self.instance_variable_set( k, v )
          @__atttributes__ |= k
        else
          self.instance_variable_set( "@#{k}", v )
          @__atttributes__ |= "@#{k}"
          #self.send( "#{k}=", v ) #if respond_to?("#{k}=")
        end
      end
    end
  end
  
  # Set setter methods and/or instance vars using a hash from another object.
  #
  #   require 'facet/object/assign_with'
  #
  #   class O
  #     attr_accessor :d
  #     def initialize( a, b, c, d)
  #       @a = a
  #       @@b = b
  #       $c = c
  #       d = d
  #     end
  #   end
  #   o1 = O.new(1,2,3,4)
  #   o2 = O.new(0,0,0,0)
  #
  #   o2.assign_from { o1, '@a', '@@b', '$c', 'd' }
  #   o2.instance_eval{ @a }   #=> 1
  #   o2.instance_eval{ @@b }  #=> 2
  #   o2.instance_eval{ $c }   #=> 3
  #   02.d                     #=> 4
  #
  # See also #assign_with.
  def assign_from(obj, *fields)
    fields.each do |k|
      var = k.to_s
      var = var.slice(0,1) == '@' ? var : "@#{var}"
      self.instance_variable_set( var, obj.instance_variable_get(var) ) if obj.instance_variables.include?(var)
      @__atttributes__ |= var
#       else
#         self.instance_variable_set( k, obj.instance_variable_get(k) ) if obj.instance_variables.include?(k)
#         #self.send( "#{k}=", obj.send("#{k}") ) if self.respond_to?("#{k}=") && obj.respond_to?("#{k}")
#       end
    end
  end
end
