require 'facet/array/at_rand'
class String

  # Return a random separation of the string.
  # Default separator is <tt>$/</tt>.
  #
  #   require 'facet/string/at_rand'
  #
  #   "Ruby rules".at_rand(1, ' ')  #=> ["Ruby"]
  #
  def at_rand(n=nil, separator=$/)
    separator = /\n+/ if separator == ''
    separator = /\s+/ unless separator
    n = n.to_i
    self.split(separator,-1).at_rand(n)
  end

  # Return a random separation while removing it
  # from the string. Default separator is <tt>$/</tt>.
  #
  #   require 'facet/string/at_rand'
  #
  #   "Ruby rules".at_rand(1, ' ')  #=> ["Ruby"]
  #
  def at_rand!(n=nil)
    separator = /\n+/ if separator == ''
    separator = /\s+/ unless separator
    n = n.to_i
    s,e = 0,0
    n.times { e = index( separator, s ) ;  s = e+1 }
    r = self[s...e]
    self[s...e] = ''
    return r
  end

  # Return a random byte of _self_.
  #
  #   require 'facet/string/at_rand'
  #
  #   "Ruby rules".rand_byte  #=> 121
  #
  def rand_byte
    self[rand_index]
  end

  # Destructive pick_byte.  Delete a random byte of _self_ and return it.
  #
  #   require 'facet/string/at_rand'
  #
  #   s = "Ruby rules"
  #   s.pick_byte!  #=> 121
  #   s             #=> "Rub rules"
  #
  def rand_byte!
    i = rand_index
    rv = self[i,1]
    self[i,1] = ''
    rv
  end

  # Return a single-character string of a random character in _self_.
  #
  #   require 'facet/string/at_rand'
  #
  #   "Ruby rules".rand_char  #=> "y"
  #
  def rand_char
    split(//,-1).at_rand
  end

  # Destructive pick_char.  Delete a random character of the string
  # and return it as a single-character string.
  #
  #   require 'facet/string/at_rand'
  #
  #   s = "Ruby rules"
  #   s.pick_char!  #=> "y"
  #   s             #=> "Rub rules"
  #
  def rand_char!
    c = split(//,-1)
    i = c.rand_index
    r = c[ri]
    c[ri] = ''
    self.replace( c.join('') )
    return r
  end

  # Return a random string index.
  #
  #   require 'facet/string/at_rand'
  #
  #   "Ruby rules".rand_index  #=> 3
  #
  def rand_index
    rand(size)
  end
  
end
