class String
  # The #format mehtod is a special toolbox method, or methodbox.
  # Within it are a collection of finer methods.
  #
  #   require 'facet/string/format'
  #
  #   "dogthispony".format.camelcase  #=> "DogThisPony"
  #
  def format
    Format.new( self )
  end
  
  class Format
    def initialize( word )
      @self = word
    end

    # only available via public call
    def self
      @self
    end

    def camelcase #(lower_case_and_underscored_word)
      @self.to_s.gsub(/(^|_)(.)/){$2.upcase}
    end
    alias :camelize :camelcase
    
    def underscore #(camel_cased_word)
      @self.to_s.gsub(/([A-Z]+)([A-Z])/,'\1_\2').gsub(/([a-z])([A-Z])/,'\1_\2').downcase
    end
  
    def humanize #(lower_case_and_underscored_word)
      @self.to_s.gsub(/_/, " ").capitalize
    end
  
    def demodulize #(class_name_in_module)
      @self.to_s.gsub(/^.*::/, '')
    end
  
    def tableize #(class_name)
      @self.underscore.inflect.plural
    end
    
    def classify #(table_name)
      @self.inflect.singular.format.camel
    end
  
    def foreign_key(separate_class_name_and_id_with_underscore = true)
      @self.demodulize.underscore + ( separate_class_name_and_id_with_underscore ? "_id" : "id")
    end

  end
  
end


# --- dev testing ---

if $0 == __FILE__

  p "dog_this_pony".format.camelize

end

