class String
 
  def inflect
    Inflect.new( self )
  end

  class Inflect
    def initialize( word )
      @self = word
    end

    # only available via public call
    def self
      @self
    end
   
    def pluralize
      result = word.to_s.dup
      plural_rules.each do |(rule, replacement)|
        break if result.gsub!(rule, replacement)
      end
      return result      
    end

    def singularize #(word)
      result = word.to_s.dup
      singular_rules.each do |(rule, replacement)|
        break if result.gsub!(rule, replacement)
      end
      return result
    end

    private
    
    def plural_rules #:doc:
      [
        [/(x|ch|ss|sh)$/, '\1es'],            # search, switch, fix, box, process, address
        [/([^aeiouy]|qu)ies$/, '\1y'],
        [/([^aeiouy]|qu)y$/, '\1ies'],        # query, ability, agency
        [/(?:([^f])fe|([lr])f)$/, '\1\2ves'], # half, safe, wife
        [/sis$/, 'ses'],                      # basis, diagnosis
        [/([ti])um$/, '\1a'],                 # datum, medium
        [/person$/, 'people'],                # person, salesperson
        [/man$/, 'men'],                      # man, woman, spokesman
        [/child$/, 'children'],               # child
        [/s$/, 's'],                          # no change (compatibility)
        [/$/, 's']
      ]
    end

    def singular_rules #:doc:
      [
        [/(x|ch|ss)es$/, '\1'],
        [/movies$/, 'movie'],
        [/([^aeiouy]|qu)ies$/, '\1y'],
        [/([lr])ves$/, '\1f'],
        [/([^f])ves$/, '\1fe'],
        [/(analy|ba|diagno|parenthe|progno|synop|the)ses$/, '\1sis'],
        [/([ti])a$/, '\1um'],
        [/people$/, 'person'],
        [/men$/, 'man'],
        [/status$/, 'status'],
        [/children$/, 'child'],
        [/s$/, '']
      ]
    end
       
  end

end
