require 'time'  # requires time extension

class Time

  MONTH_NAMES = %w| January February March April May June July August September October November December |
  DAY_NAMES   = %w| Sunday Monday Tuesday Wednesday Thursday Friday Saturday |

  # Replaces every token in _formatString_ with its corresponding value. See the table below.
  #
  #    token:     description:             example:
  #    #YYYY#     4-digit year             2004
  #    #YY#       2-digit year             04
  #    #MMMM#     full month name          February
  #    #MMM#      3-letter month name      Feb
  #    #MM#       2-digit month number     02
  #    #M#        month number             2
  #    #DDDD#     full weekday name        Wednesday
  #    #DDD#      3-letter weekday name    Wed
  #    #DD#       2-digit day number       09
  #    #D#        day number               9
  #    #th#       day ordinal suffix       nd
  #    #hhh#      military/24-based hour   17
  #    #hh#       2-digit hour             05
  #    #h#        hour                     5
  #    #mm#       2-digit minute           07
  #    #m#        minute                   7
  #    #ss#       2-digit second           09
  #    #s#        second                   9
  #    #ampm#     "am" or "pm"             pm
  #    #AMPM#     "AM" or "PM"             PM
  #
  # Non-tokens are left untouched in the output string.
  #
  #   require 'facet/time/custom_format'
  #
  #   time = Time.parse('2/3/2004 15:37')
  #   time.custom_format('#DDDD#, #MMMM# #D##th# @ #h#:#mm##ampm#')
  #      #=>  'Tuesday, February 3rd @ 3:37pm'
  #   time.custom_format('#YYYY#-#MMM#-#D#')
  #      #=>  '2004-Feb-3'
  #   time.custom_format('#MM#/#DD#/#YY#')
  #      #=>  '02/03/04'
  #
  def custom_format(formatString)
    zYY   = (zYYYY=self.year.to_s)[-2..-1]
    zMM   = (zM=self.mon)<10 ? ('0'+zM.to_s) : zM.to_s
    zMMM  = (zMMMM=MONTH_NAMES[zM-1])[0...3]
    zDD   = (zD=self.day)<10 ? ('0'+zD.to_s) : zD.to_s
    zDDD  = (zDDDD=DAY_NAMES[wday])[0...3]
    zth   = (zD>=10&&zD<=20) ? 'th' : ((dMod=zD%10)==1) ? 'st' : (dMod==2) ? 'nd' : (dMod==3) ? 'rd' : 'th'
  
    zh    = (zhhh=self.hour)==0 ? 24 : zhhh; zh-=12 if (zh>12)
    zhh   = zh<10 ? ('0'+zh.to_s) : zh.to_s
    zmm   = (zm=self.min)<10 ? ('0'+zm.to_s) : zm.to_s
    zss   = (zs=self.sec)<10 ? ('0'+zs.to_s) : zs.to_s
    zAMPM = (zampm=zhhh<12 ? 'am' : 'pm').upcase

    f=formatString
    f.gsub!(/#YYYY#/,zYYYY)
    f.gsub!(/#YY#/,zYY)
    f.gsub!(/#MMMM#/,zMMMM)
    f.gsub!(/#MMM#/,zMMM)
    f.gsub!(/#MM#/,zMM.to_s)
    f.gsub!(/#M#/,zM.to_s)
    f.gsub!(/#DDDD#/,zDDDD)
    f.gsub!(/#DDD#/,zDDD)
    f.gsub!(/#DD#/,zDD)
    f.gsub!(/#D#/,zD.to_s)
    f.gsub!(/#th#/,zth)
    f.gsub!(/#hhh#/,zhhh.to_s)
    f.gsub!(/#hh#/,zhh)
    f.gsub!(/#h#/,zh.to_s)
    f.gsub!(/#mm#/,zmm)
    f.gsub!(/#m#/,zm.to_s)
    f.gsub!(/#ss#/,zss)
    f.gsub!(/#s#/,zs.to_s)
    f.gsub!(/#ampm#/,zampm)
    f.gsub!(/#AMPM#/,zAMPM);f
  end

end


=begin :nodoc:
# --- devlopment testing ---

if $0 == __FILE__

  #require 'test/unit'
  
  #require 'raspberry/atom/time/custom_format'
  
  class TC_Time < Test::Unit::TestCase
  
    # custom_format
    
    def test_custom_format
    p Time.methods.sort
      time = Time::parse('2/3/2004 15:37')
      cf = time.custom_format('#DDDD#, #MMMM# #D##th# @ #h#:#mm##ampm#')
      assert_equal( 'Tuesday, February 3rd @ 3:37pm', cf )
      cf = time.custom_format('#YYYY#-#MMM#-#D#')
      assert_equal( '2004-Feb-3', cf )
      cf = time.custom_format('#MM#/#DD#/#YY#')
      assert_equal( '02/03/04', cf )
    end
    
  end

end
=end
