
require 'test/unit'

require "facet/binding/eval"
require "facet/binding/self"
require "facet/binding/caller"
require "facet/binding/defined?"
require "facet/binding/called"
require "facet/binding/local_variables"
require "facet/binding/lineno"
require "facet/binding/delegate!"


class TC_Binding < Test::Unit::TestCase

  def setup
    a = 1
    b = 2
    x = "hello"
    # the line number must be updated if it moves
    @bind = binding; @this_line_no = 21
  end

  # eval

  def test_eval
    assert_equal( "hello", @bind.eval("x") )
  end

  def test_filename
    assert_equal( 'tc_binding.rb', File.basename( @bind.filename ) )
  end

  def test_lineno
    assert_equal( @this_line_no, @bind.lineno )
  end

  # called
  
  def test_called
    assert_equal( :setup, @bind.called )
  end

  # class (not really a test of Facets)
  
  def test_class
    assert_equal( TC_Binding, @bind.self.class )
  end
  
  # defined?
  
  def test_defined?
    assert( @bind.defined?("a") )
  end
  
  # local variables
  
  def test_loacal_variables
    assert_equal( ["a","b","x"], @bind.local_variables )
  end
  
  def test_brackets
    assert_equal( "hello", @bind["x"] )
  end
  
  def test_brackets_eq
    assert_nothing_raised{ @bind["x"] = "goodbye" }
    assert_equal( "goodbye", @bind["x"] )
  end
  
  # self
  
  def test_self
    assert_equal( self, @bind.self )
  end

end
