
require 'test/unit'

require 'facet/enumerable/collect_with_counter'
require 'facet/enumerable/cross'
require 'facet/enumerable/each_slice'
require 'facet/enumerable/every'
require 'facet/enumerable/graph'
require 'facet/enumerable/per'
require 'facet/enumerable/none?'
require 'facet/enumerable/one?'
require 'facet/enumerable/partition_by'
require 'facet/enumerable/to_hash'
require 'facet/enumerable/uniq_by'


class TC_Enumerable < Test::Unit::TestCase

  # collect_with_counter
  
  def test_collect_with_counter
    a = [1,2,3].collect_with_counter{ |e,i| e*i }
    assert_equal( [0,2,6], a )
    a = [1,2,3].collect_with_index{ |e,i| e+i }
    assert_equal( [1,3,5], a )
  end
  
  # cross
  
  def test_cross
    i = [[1,2], [3,4]]
    o = [[1, 3], [1, 4], [2, 3], [2, 4]]
    assert_equal( o, Enumerable.cross(*i) )
    i = [[1,2], [4], ["apple", "banana"]]
    o = [[1, 4, "apple"], [1, 4, "banana"], [2, 4, "apple"], [2, 4, "banana"]]
    assert_equal( o, Enumerable.cross(*i) )
    a = []
    [1,2,3].cross([4,5,6]) {|elem| a << elem }
    assert_equal( [[1, 4],[1, 5],[1, 6],[2, 4],[2, 5],[2, 6],[3, 4],[3, 5],[3, 6]], a )
  end
  
  # each_slice
  
  def test_each_slice
    x = []
    [1,2,3,4].each_slice{ |a,b| x << [a,b] }
    o = [[1,2],[3,4]]
    assert_equal( o, x )

    x = []
    [1,2,3,4,5].each_by{ |a,b,c| x << [a,b,c] }
    o = [[1,2,3],[4,5,nil]]
    assert_equal( o, x )

    x = []
    [1,2,3,4].each_slice(2){ |a,b| x << [a,b] }
    o = [[1,2],[3,4]]
    assert_equal( o, x )

    x = []
    [1,2,3,4,5,6,7,8].each_slice(4){ |*a| x << a }
    o = [[1,2,3,4],[5,6,7,8]]
    assert_equal( o, x )
    
    x = []
    [1,2,3,4,5,6].each_slice(3){ |*a| x << a }
    o = [[1,2,3],[4,5,6]]
    assert_equal( o, x )
    
    x = []
    [1,2,3,4].each_by(2){ |a,b| x << [a,b] }
    o = [[1,2],[3,4]]
    assert_equal( o, x )
    
    x = []
    [1,2,3,4,5].each_by(3){ |*a| x << a }
    o = [[1,2,3],[4,5]]
    assert_equal( o, x )
    
    a = [1,2,3,4,5,6]
    r = []
    a.each_slice(2){ |x,y| r << [x,y] }
    assert_equal( [[1,2],[3,4],[5,6]], r )
    
    r = []
    a.each_slice(3){ |*e| r << e }
    assert_equal( [[1,2,3],[4,5,6]], r )
  
  end
  
  # every
  
  def test_every
    a = [1,2,3]
    b = [4,5]
    assert_equal( [4,5,6], a.every + 3 )
    assert_equal( [5,7], a.per + b )
    assert_equal( [2,4,6], a.% + a )
    assert_equal( [[5,7],[3,4,5]], a.%.+(b,2) )
    assert_equal( [[5,7],[4,5,6]], a.%.+(b,3) )
  end
  
  # graph
  
  def test_graph
    numbers  = (1..3)
    squares  = numbers.graph{ |n| [n, n*n] }   
    assert_equal( {1=>1, 2=>4, 3=>9}, squares )
    sq_roots = numbers.graph{ |n| [n*n, n] }
    assert_equal( {1=>1, 4=>2, 9=>3}, sq_roots )
  end

  # none?
  
  def test_none?
    a = [nil, nil]
    assert( a.none? )
    a = [false, false]
    assert( a.none? )
    a = [true, false]
    assert( ! a.none? )
    a = [nil, 1]
    assert( ! a.none? )
  end
  
  # one?
  
  def test_one?
    a = [nil, true]
    assert( a.one? )
    a = [true, false]
    assert( a.one? )
    a = [true, true]
    assert( ! a.one? )
    a = [true, 1]
    assert( ! a.one? )
    a = [1, 1]
    assert( ! a.one? )
  end
  
  # partition_by
  
  def partition_by
    x = (1..5).partition_by{ |n| n % 3 }
    o = { 0 => [3], 1 => [1, 4], 2 => [2,5] }
    assert_equal( o, x )
    x = ["I had", 1, "dollar and", 50, "cents"].partition_by { |e| e.class }
    o = { String => ["I had","dollar and","cents"], Fixnum => [1,50] }
    assert_equal( o, x )
  end

  # to_hash
    
  def test_to_hash
    a = [:a,:b,:c]
    assert_equal( { 0=>:a, 1=>:b, 2=>:c }, a.to_hash )
  end

  # uniq_by
  def test_uniq_by
    a = [-5, -4, -3, -2, -1, 0]
    r = (-5..5).to_a.uniq_by{|i| i*i }
    assert_equal( a, r )
  end
  
end
