
require 'test/unit'

require 'facet/numeric/bound_by'
require 'facet/numeric/bytes'
require 'facet/numeric/distance'
require 'facet/numeric/octet_units'
require 'facet/numeric/succ'
require 'facet/numeric/times'
require 'facet/numeric/to_b'


class TC_Numeric < Test::Unit::TestCase

  # bound_by

  def test_bound_by
    assert_equal( 6, 7.bound_by(3,6) )
    assert_equal( 12, 12.bound_by(6,13) )
  end
  
  def test_at_least
    assert_equal( 6, 4.at_least(6) )
    assert_equal( 12, 12.at_least(6) )
  end

  def test_at_most
    assert_equal( 4, 4.at_most(6) )
    assert_equal( 6, 12.at_most(6) )  
  end

  # bytes

  def test_bits
    assert_equal( 1, 8.bits )
  end
  
  def test_bytes
    assert_equal( 1024, 1024.bytes )
  end
  
  def test_kilobytes
    assert_equal( 1024**1, 1.kilobytes )
  end

  def test_megabytes
    assert_equal( 1024**2, 1.megabytes )
  end
  
  def test_gigabytes
    assert_equal( 1024**3, 1.gigabytes )
  end
  
  def test_terabytes
    assert_equal( 1024**4, 1.terabytes )
  end
  
  # distance
  
  def test_distance
    assert_equal( 4, 10.distance(6) )
    assert_equal( 2, 10.distance(8) )
    assert_equal( -2, 7.distance(9) )
  end

  # octet_units
  
  def test_octet_units
    assert_equal( "1.00 KB", 1024.octet_units )
    assert_equal( "1.00 MB", 1048576.octet_units )
    assert_equal( "1.00 GB", 1073741824.octet_units )
    assert_equal( "1.00 TB", 1099511627776.octet_units )
  end
  
  # succ
  
  def test_succ
    assert_equal(  5,  4.succ )
    assert_equal( -1, -2.succ )
    assert_equal(  6,  4.succ(2) )
    assert_equal(  0, -2.succ(2) )
    assert_equal(  2,  4.succ(-2) )
    assert_equal( -4, -2.succ(-2) )
  end

  # times
  
  def test_microseconds
    assert_equal( 0.000001, 1.microsecond )
  end
  
  def test_milliseconds
    assert_equal( 0.001, 1.millisecond )
  end
  
  def test_seconds
    assert_equal( 60**0, 1.seconds )
  end

  def test_minutes
    assert_equal( 60**1, 1.minutes )
  end

  def test_hours
    assert_equal( 60**2, 1.hours )
  end
  
  def test_days
    assert_equal( 24*(60**2), 1.days )
  end
  
  def test_weeks
    assert_equal( 7*24*(60**2), 1.weeks )
  end
  
  def test_fortnights
    assert_equal( 14*24*(60**2), 1.fortnights )
  end
  
  def test_months
    assert_equal( 30*24*(60**2), 1.months )
  end
  
  def test_years
    assert_equal( 365*24*(60**2), 1.years )
  end

  def test_before
    t = Time.now
    assert_equal( t - 1.day, 1.day.before(t) )
  end

  def test_after
    t = Time.now
    assert_equal( t + 1.day, 1.day.after(t) )
  end

  # to_b
  
  def test_to_b
    assert_equal( false, 0.to_b )
    assert_equal( true, 1.to_b )
  end
  
end
