
require 'test/unit'

require 'facet/object/assign_with'
require 'facet/object/bool?'
require 'facet/object/__class__'
require 'facet/object/copy'
require 'facet/object/in?'
require 'facet/object/object_hexid'
require 'facet/object/set_with'
require 'facet/object/special_class'
require 'facet/object/to_b'
require 'facet/object/with_accessor'


class TC_Object < Test::Unit::TestCase

  # assign_with

  def test_assign_with
    ahash = { "z"=>0, "@a"=>1, "@b"=>2, "@@a"=>3, "$a"=>4 }
    assign_with( ahash )
    assert_equal( 0, @z )
    assert_equal( 1, @a )
    assert_equal( 2, @b )
    assert_equal( 3, @@a )
    assert_equal( 4, $a )
  end
  
  def test_assign_from
    o = Object.new
    o.instance_eval{ @z=0; @a=1; @b=2; @@a=3; $a=4 }
    assign_from( o, "z", "@a", "@b", "@@a", "$a" )
    assert_equal( 1, @a )
    assert_equal( 2, @b )
    assert_equal( 3, @@a )
    assert_equal( 4, $a )
  end
  
  # bool?
  
  def test_bool?
    assert_equal( true, true.bool? )
    assert_equal( true, false.bool? )
    assert_equal( false, nil.bool? )
    assert_equal( false, 0.bool? )
  end
  
  def test_true?
    assert( true.true? )
    assert( (1 == 1).true? )
    assert( ! (1 == 2).true? )  
  end

  def test_false?
    assert( false.false? )
    assert( (1 == 2).false? )
    assert( ! (1 == 1).false? )
  end

  # __class__
  
  def test__class__
    assert_equal( self.class, self.__class__ )
    assert_equal( self.class, self.object_class )
  end

  # copy

  def test_copy
    # TO DO
  end

  # in?

  def test_in?
    assert( 5.in?(0..10) )
    assert( 5.in?([1,2,3,4,5]) )
  end

  # object_hexid
  
  def test_object_hexid
    o = Object.new
    assert( o.inspect.index(o.object_hexid) )
  end

  # set_with

  Customer = Struct.new( "Customer", :name, :address, :zip )
  
  def test_set_with
    bob = Customer.new()
    sethash = { :name => "Bob Sawyer", :address => "123 Maple, Anytown NC", :zip => 12345 }
    bob.set_with(sethash)
    assert_equal("Bob Sawyer", bob.name)
    assert_equal("123 Maple, Anytown NC", bob.address)
    assert_equal(12345, bob.zip)
  end  
  
  def test_set_from
    bob = Customer.new("Bob Sawyer", "123 Maple, Anytown NC", 12345)
    joe = Customer.new("Joe Pitare")
    joe.set_from(bob, :address, :zip)
    assert_equal("Joe Pitare", joe.name)
    assert_equal("123 Maple, Anytown NC", joe.address)
    assert_equal(12345, joe.zip)  
  end
  
  # special_class
  
  def test_special_class
    o = Object.new
    assert_equal( (class << o; self; end), o.special_class )
    assert_nothing_raised do
      o.special_class_eval{ @@a = "test" }
    end
    assert_nothing_raised do
      o.define_special_method(:testing){ |x| x + 1 }
    end
    assert_equal(2, o.testing(1) )
  end
  
  # to_b
  
  def test_to_b
    assert_equal( true, true.to_b )
    assert_equal( false, false.to_b )
    assert_equal( false, nil.to_b )
  end
  
  # with_accessor
  
  #def test_with_accessor
  #  o
  #end
  
end
